# -*- coding: utf-8 -*-
from openerp import api, fields, models
from openerp.tools.translate import _
from openerp.tools import DEFAULT_SERVER_DATETIME_FORMAT, DEFAULT_SERVER_DATE_FORMAT
from pytz import timezone
from datetime import datetime,timedelta
import logging
_logger = logging.getLogger(__name__)

class EiruStockReturnPicking(models.TransientModel):
    _inherit = 'stock.return.picking'

    ''' Timezone '''
    def get_timezone(self):
        return timezone(self._context.get('tz') or self.env.user.tz)

    ''' Datetime '''
    def get_datetime(self):
        return datetime.now(self.get_timezone()).strftime(DEFAULT_SERVER_DATETIME_FORMAT)

    ''' Date '''
    def get_date(self):
        return datetime.now(self.get_timezone()).strftime(DEFAULT_SERVER_DATE_FORMAT)

    '''
           ____ _____ _____   ____ ___ ____ _  _____ _   _  ____
          / ___| ____|_   _| |  _ \_ _/ ___| |/ /_ _| \ | |/ ___|
         | |  _|  _|   | |   | |_) | | |   | ' / | ||  \| | |  _
         | |_| | |___  | |   |  __/| | |___| . \ | || |\  | |_| |
          \____|_____| |_|   |_|  |___\____|_|\_\___|_| \_|\____|
    '''
    ''' GET Stock picking '''
    @api.model
    def eiru_get_stock_picking_order(self, idOrder, model):
        pickingLine = []
        pickingReturn = []
        ''' decimal.precision '''
        decimalPrecision = self.env['decimal.precision'].precision_get('Product UoS')

        ''' sale.order '''
        order =self.env[model].browse(idOrder)
        if (not order):
            return {
                'state': False,
                'message': "No fue posible obtener la orden."
            }

        procurement = self.env['procurement.group'].search([('name', '=', order.name)])
        if (not procurement):
            return {
                'state': False,
                'message': "No fue posible localizar la petición de abastecimiento."
            }
        ''' Get stock.picking '''
        pickingSale = self.env['stock.picking'].search([('group_id.id', '=', procurement.id),('origin', '=', order.name)],order='id')
        if (not pickingSale):
            return {
                'state': False,
                'message': "Ya existe mas de un movimiento de stock de este pedido."
            }

        moveIds = map(lambda x: x.id, pickingSale.move_lines)
        for line in self.env['stock.move'].search([('id', 'in', moveIds)]):
            productProduct = self.env['product.product'].browse(line.product_id.id)
            if (not productProduct):
                continue


            default_code = str(productProduct.ean13) if productProduct.ean13 else ""
            name = str(productProduct.name) if productProduct.name else ""
            productName = default_code + " " + name

            pickingLine.append({
                'moveId': line.id,
                'qty_order': round((line.product_uom_qty) ,decimalPrecision),
                'qty_return': 0,
                'qty': round((line.product_uom_qty) ,decimalPrecision),
                'productId' : productProduct.id,
                'productName': productName,
            })

        if (not pickingLine):
            return {
                'state': False,
                'message': "Ya existe mas de un movimiento de stock de este pedido."
            }

        stockPicking = self.env['stock.picking'].search([('group_id.id', '=', procurement.id), ('origin', '=', pickingSale.name)],order='id')

        for picking in stockPicking:
            moveIds = map(lambda x: x.id, picking.move_lines)
            for line in self.env['stock.move'].search([('id', 'in', moveIds)]):
                productProduct = self.env['product.product'].browse(line.product_id.id)
                if (not productProduct):
                    continue

                moveIds = map(lambda x: x['moveId'], pickingLine)
                if (line.origin_returned_move_id.id in moveIds):
                    for stock in pickingLine:
                        if (stock['moveId'] == line.origin_returned_move_id.id):
                            stock['qty_return'] += line.product_uom_qty

        if (not pickingLine):
            return {
                'state': False,
                'message': "Ya existe mas de un movimiento de stock de este pedido."
            }

        for stock in pickingLine:
            stock['qty'] = round((stock['qty'] - (stock['qty_return'])) ,decimalPrecision),
            stock['qty_order'] = round((stock['qty_order']) ,decimalPrecision),
            stock['qty_return'] = round((stock['qty_return']) ,decimalPrecision),

        pickingReturn.append({
            'order': [{
                'id':order.id,
                'name':order.name,
                'dateOrder': order.date_order,
            }],
            'picking': [{
                'id': pickingSale.id,
                'name': pickingSale.name,
                'state': pickingSale.state,
                'locationOrigen': pickingSale.location_id.id,
                'locationDest': pickingSale.location_dest_id.id,
                'productFormat': {
                    'thousandsSeparator': '.',
                    'decimalSeparator': ',',
                    'decimalPlaces': decimalPrecision
                },
                'lines': pickingLine
            }],
        })

        return {
            'state': True,
            'pickingOrder': pickingReturn
        }

    '''
          ____  _____ _____ _   _ ____  _   _   ____ ___ ____ _  _____ _   _  ____
         |  _ \| ____|_   _| | | |  _ \| \ | | |  _ \_ _/ ___| |/ /_ _| \ | |/ ___|
         | |_) |  _|   | | | | | | |_) |  \| | | |_) | | |   | ' / | ||  \| | |  _
         |  _ <| |___  | | | |_| |  _ <| |\  | |  __/| | |___| . \ | || |\  | |_| |
         |_| \_\_____| |_|  \___/|_| \_\_| \_| |_|  |___\____|_|\_\___|_| \_|\____|
    '''
    ''' create_stock_return '''
    # @api.multi
    def create_stock_return(self, vals):
        return super(EiruStockReturnPicking, self).create(vals)

    ''' eiru_create_stock_return_picking '''
    @api.model
    def eiru_create_stock_return_picking(self, values):
        _logger.info('Stock Picking')
        fields = ['product_return_moves', 'move_dest_exists', 'invoice_state']
        res = super(EiruStockReturnPicking, self).default_get(fields)

        stockMove = []
        for line in values['move']:
            stockMove.append([0, False,{
                'product_id': line['productId'],
                'move_id': line['moveID'],
                'quantity':  line['qty']
            }])

        if 'product_return_moves' in fields:
            res.update({'product_return_moves': stockMove})

        if 'move_dest_exists' in fields:
            res.update({'move_dest_exists': False})

        if 'invoice_state' in fields:
            res.update({'invoice_state': '2binvoiced'})

        ''' stock.return.picking '''
        pickingReturn = self.env['stock.return.picking'].create_stock_return(res)
        new_picking_id, pick_type_id = pickingReturn.with_context(active_id=values['pickingId'],active_model='stock.picking')._create_returns()

        ''' Get Picking '''
        stockPicking = self.env['stock.picking'].browse(new_picking_id)
        if(not stockPicking):
            return {
                'state': True,
                'message': 'No fue posible localizar la devolución'
            }

        if (values['model'] == 'purchase.order'):
            for moveline in stockPicking.move_lines:
                moveOriginal = self.env['stock.move'].browse(moveline.origin_returned_move_id.id)
                if (not moveOriginal):
                    return False

                orederLine = self.env['purchase.order.line'].browse(moveOriginal.purchase_line_id.id)
                if (not orederLine):
                    return False

                moveline.write({
                    'purchase_line_id': orederLine.id
                })

        ''' Transferir producto '''
        stockPicking.force_assign()
        stockPicking.action_done()

        if (values['invoiced']):
            ''' Crear Factura de la devolucion '''
            stockInvoice = self.env['stock.invoice.onshipping'].with_context(active_ids=[stockPicking.id]).eiru_create_stock_invoice_onshipping(self.get_date())
            if (not stockInvoice):
                return {
                    'state': True,
                    'message': 'No es posible crear la factura'
                }

            if (values['model'] == 'purchase.order'):
                purchase = self.env[values['model']].browse(values['orderID'])
                if (not purchase):
                    return {
                        'state': True,
                        'message': 'No es posible localizar la compra.'
                    }

                invoicesIds = map(lambda x: x.id, purchase.invoice_ids)
                invoicesIds.append(stockInvoice[0])
                invoice = self.env['account.invoice'].search([('id', 'in', stockInvoice)])
                purchase.write({'invoice_ids': [(4, invoice.id, False)]})

        return {
            'state': True,
            'message': 'Devolución realizada con suceso'
        }

''' stock.invoice.onshipping '''
class EiruStockInvoiceOnshipping(models.TransientModel):
    _inherit = 'stock.invoice.onshipping'

    ''' Open Invoice '''
    @api.multi
    def create_invoice(self):
        res = super(EiruStockInvoiceOnshipping, self).create_invoice()
        invoice = self.env['account.invoice'].search([('id', 'in', res)])
        if (invoice):
            invoice.signal_workflow('invoice_open')
        return res

    ''' create Invoices stock returned '''
    def eiru_create_stock_invoice_onshipping(self, dateServer):
        ''' Get Journal Type '''
        journalType = self._get_journal_type()
        ''' Get Journal '''
        journal = self._get_journal()

        ''' Create stock.invoice.onshipping '''
        res = {
            'journal_id': journal,
            'journal_type': journalType,
            'invoice_date': dateServer,
        }
        newStockInvoice = self.create(res)

        ''' Create Invoice '''
        res = newStockInvoice.create_invoice()

        return res
